@@ -57,6 +57,7 @@ def cli():
     pass
 
 
+# Generate
 @cli.command()
 @click.option(
     "-i",
@@ -151,7 +152,7 @@ def generate(
     filter_duplicate,
     fragment_count,
     mapping_quality,
-    threads
+    threads,
 ):
     """Command that helps to generate genotyped MAF,
     the output file will be labelled with
@@ -300,6 +301,9 @@ def generate_gbcms_cmd(
     mapping_quality,
     threads,
 ):
+
+    """This will help generate command for GetBaseCountMultiSample"""
+
     sample_id = patient_id + "-" + btype
     output_maf = pathlib.Path.cwd().joinpath(sample_id + "_genotyped.maf")
 
@@ -329,6 +333,7 @@ def generate_gbcms_cmd(
     return (cmd, output_maf)
 
 
+# Merge
 @cli.command()
 @click.option(
     "-i",
@@ -369,10 +374,10 @@ def generate_gbcms_cmd(
 def merge(
     patient_id, input_maf, input_standard_maf, input_duplex_maf, input_simplex_maf
 ):
-    """ 
-    Given original input MAF used as an input for GBCMS along with 
-    GBCMS generated output MAF for standard_bam, duplex_bam or simplex bam, 
-    Merge them into a single output MAF format. 
+    """
+    Given original input MAF used as an input for GBCMS along with
+    GBCMS generated output MAF for standard_bam, duplex_bam or simplex bam,
+    Merge them into a single output MAF format.
     If both duplex_bam and simplex_bam based MAF are provided
     the program will generate merged genotypes as well.
     The output file will be based on the give alphanumeric patient identifier as prefix.
@@ -415,21 +420,34 @@ def merge(
         logger.info(
             "genotype_variants:small_variants:merge:: Original MAF -> %s", input_maf
         )
-        o_maf = pd.read_csv(input_maf, sep="\t", header="infer")
+        o_maf = pd.read_csv(input_maf, sep="\t", header="infer", index_col=False)
     if input_standard_maf:
         logger.info(
             "genotype_variants:small_variants:merge:: STANDARD BAM MAF -> %s",
             input_standard_maf,
         )
-        i_maf = pd.read_csv(input_standard_maf, sep="\t", header="infer")
+
+        create_empty_maf_if_missing(input_standard_maf)
+
+        i_maf = pd.read_csv(
+            input_standard_maf, sep="\t", header="infer", index_col=False
+        )
     if input_duplex_maf:
-        d_maf = pd.read_csv(input_duplex_maf, sep="\t", header="infer")
+
+        create_empty_maf_if_missing(input_duplex_maf)
+
+        d_maf = pd.read_csv(input_duplex_maf, sep="\t", header="infer", index_col=False)
         logger.info(
             "genotype_variants:small_variants:merge:: DUPLEX BAM MAF -> %s",
             input_duplex_maf,
         )
     if input_simplex_maf:
-        s_maf = pd.read_csv(input_simplex_maf, sep="\t", header="infer")
+
+        create_empty_maf_if_missing(input_simplex_maf)
+
+        s_maf = pd.read_csv(
+            input_simplex_maf, sep="\t", header="infer", index_col=False
+        )
         logger.info(
             "genotype_variants:small_variants:merge:: SIMPLEX BAM MAF -> %s",
             input_simplex_maf,
@@ -440,10 +458,14 @@ def merge(
 
     if d_maf is not None and s_maf is not None:
         ds_maf = cdsd(s_maf, d_maf)
+        file_name = pathlib.Path.cwd().joinpath(
+            patient_id + "-SIMPLEX-DUPLEX" + "_genotyped.maf"
+        )
+        write_csv(file_name, ds_maf)
 
     # generate data frame based on satisfying conditions
     file_name = None
-    (df_o_s_ds, df_s_ds, df_s_ds) = None, None, None
+    (df_o_s_ds, df_o_s, df_s_ds, df_s_ds) = None, None, None, None
     if o_maf is not None and i_maf is not None and ds_maf is not None:
         df_o_s_ds = camd(o_maf, i_maf, ds_maf)
         file_name = pathlib.Path.cwd().joinpath(
@@ -451,7 +473,11 @@ def merge(
         )
         write_csv(file_name, df_o_s_ds)
     elif o_maf is not None and i_maf is not None:
-        pass
+        df_o_s = camd(o_maf, i_maf, None)
+        file_name = pathlib.Path.cwd().joinpath(
+            patient_id + "-ORG-STD" + "_genotyped.maf"
+        )
+        write_csv(file_name, df_o_s)
     elif o_maf is not None and ds_maf is not None:
         df_o_ds = camd(o_maf, None, ds_maf)
         file_name = pathlib.Path.cwd().joinpath(
@@ -486,6 +512,59 @@ def merge(
     return file_name
 
 
+def create_empty_maf_if_missing(filename):
+    header = [
+        'Hugo_Symbol',
+        'Entrez_Gene_Id',
+        'Center',
+        'NCBI_Build',
+        'Chromosome',
+        'Start_Position',
+        'End_Position',
+        'Strand',
+        'Variant_Classification',
+        'Variant_Type',
+        'Reference_Allele',
+        'Tumor_Seq_Allele1',
+        'Tumor_Seq_Allele2',
+        'dbSNP_RS',
+        'dbSNP_Val_Status',
+        'Tumor_Sample_Barcode',
+        'Matched_Norm_Sample_Barcode',
+        'Match_Norm_Seq_Allele1',
+        'Match_Norm_Seq_Allele2',
+        'Tumor_Validation_Allele1',
+        'Tumor_Validation_Allele2',
+        'Match_Norm_Validation_Allele1',
+        'Match_Norm_Validation_Allele2',
+        'Verification_Status',
+        'Validation_Status',
+        'Mutation_Status',
+        'Sequencing_Phase',
+        'Sequence_Source',
+        'Validation_Method',
+        'Score',
+        'BAM_File',
+        'Sequencer',
+        't_ref_count',
+        't_alt_count',
+        'n_ref_count',
+        'n_alt_count',
+        'Caller',
+        't_total_count',
+        't_variant_frequency',
+        't_total_count_forward',
+        't_ref_count_forward',
+        't_alt_count_forward',
+        't_total_count_fragment',
+        't_ref_count_fragment',
+        't_alt_count_fragment']
+
+    if not os.path.exists(filename):
+        empty_df = pd.DataFrame(columns=header)
+        empty_df.to_csv(filename, index=False, sep='\t')
+
+
 def write_csv(file_name, data_frame):
     try:
         data_frame.to_csv(str(file_name), sep="\t", index=False)
@@ -502,6 +581,7 @@ def write_csv(file_name, data_frame):
         exit(1)
 
 
+# All
 @cli.command()
 @click.option(
     "-i",
@@ -599,9 +679,9 @@ def all(
     threads,
 ):
     """
-    Command that helps to generate genotyped MAF and 
+    Command that helps to generate genotyped MAF and
     merge the genotyped MAF.
-    the output file will be labelled with 
+    the output file will be labelled with
     patient identifier as prefix
     """
     pid = os.getpid()
@@ -638,7 +718,9 @@ def all(
         mapping_quality,
         threads,
     )
-    final_file = merge.callback(patient_id, input_maf, standard_maf, simplex_maf, duplex_maf)
+    final_file = merge.callback(
+        patient_id, input_maf, standard_maf, duplex_maf, simplex_maf
+    )
     t1_stop = time.perf_counter()
     t2_stop = time.process_time()
     logger.info("--------------------------------------------------")
@@ -647,13 +729,15 @@ def all(
     logger.info("--------------------------------------------------")
     return final_file
 
+
+# Multiple Sample Process
 @cli.command()
 @click.option(
     "-i",
     "--input-metadata",
     required=True,
     type=click.Path(exists=True),
-    help="Full path to metadata file in TSV/EXCEL format, with following headers: patient_id, maf, standard_bam, duplex_bam, simplex_bam. Make sure to use full paths inside the metadata file",
+    help="Full path to metadata file in TSV/EXCEL format, with following headers: sample_id, maf, standard_bam, duplex_bam, simplex_bam. Make sure to use full paths inside the metadata file",
 )
 @click.option(
     "-r",
@@ -702,7 +786,7 @@ def all(
     help="Number of threads to use for GetBaseCountMultiSample",
 )
 @click_log.simple_verbosity_option(logger)
-def multiple_patient(
+def multiple_samples(
     input_metadata,
     reference_fasta,
     gbcms_path,
@@ -712,18 +796,18 @@ def multiple_patient(
     threads,
 ):
     """
-    Command that helps to generate genotyped MAF and 
-    merge the genotyped MAF for multiple patients.
-    the output file will be labelled with 
+    Command that helps to generate genotyped MAF and
+    merge the genotyped MAF for multiple samples.
+    the output file will be labelled with
     patient identifier as prefix
 
     Expected header of metadata_file in any order:
-    patient_id
-    maf
-    standard_bam
-    duplex_bam
+    sample_id,
+    maf,
+    standard_bam,
+    duplex_bam,
     simplex_bam
-    
+
     For maf, standard_bam, duplex_bam and simplex_bam please include full path to the file.
     """
     pid = os.getpid()
@@ -747,76 +831,95 @@ def multiple_patient(
     )
     t1_start = time.perf_counter()
     t2_start = time.process_time()
-    metadata = pd.DataFrame()
+    metadata = None
     try:
         metadata = pd.read_excel(input_metadata)
     except:
         e = sys.exc_info()[0]
         logger.warning(
-            "genotype_variants:small_variants:multiple_patient:: could not read to EXCEL file, due to error: %s",
+            "genotype_variants:small_variants:multiple_samples:: could not read to EXCEL file, due to error: %s",
             e,
         )
         logger.warning(
-            "genotype_variants:small_variants:multiple_patient:: Assuming its as TSV file"
+            "genotype_variants:small_variants:multiple_samples:: Assuming its as TSV file"
         )
         pass
-    try:
-        metadata = pd.read_csv(input_metadata, sep="\t", header="infer")
-    except:
-        e = sys.exc_info()[0]
-        logger.error(
-            "genotype_variants:small_variants:multiple_patient:: could not read TSV file, due to error: %s. Please fix and rerun the script",
-            e,
-        )
-        exit(1)
+    if metadata is None:
+        try:
+            metadata = pd.read_csv(input_metadata, sep="\t", header="infer")
+        except:
+            e = sys.exc_info()[0]
+            logger.error(
+                "genotype_variants:small_variants:multiple_samples:: could not read TSV file, due to error: %s. Please fix and rerun the script",
+                e,
+            )
+            exit(1)
+    else:
+        pass
     for ind in metadata.index:
-        if pd.notnull(metadata['maf'][ind]):
-            if pathlib.Path(metadata['maf'][ind]).is_file():
-                input_maf = metadata['maf'][ind]
+        if pd.notnull(metadata["maf"][ind]):
+            if pathlib.Path(metadata["maf"][ind]).is_file():
+                input_maf = metadata["maf"][ind]
             else:
-                logger.error("genotype_variants::small_variants::multiple_patient:: Maf file to genotype variants is present but the path is invalid. Please provide a valid path")
+                logger.error(
+                    "genotype_variants::small_variants::multiple_samples:: Maf file to genotype variants is present but the path is invalid. Please provide a valid path"
+                )
                 exit(1)
         else:
-            logger.error("genotype_variants::small_variants::multiple_patient:: Maf file to genotype variants is not present and is required.")
+            logger.error(
+                "genotype_variants::small_variants::multiple_samples:: Maf file to genotype variants is not present and is required."
+            )
             exit(1)
-        if pd.notnull(metadata['standard_bam'][ind]): 
-            if pathlib.Path(metadata['standard_bam'][ind]).is_file():
-                standard_bam = metadata['standard_bam'][ind]
+        if pd.notnull(metadata["standard_bam"][ind]):
+            if pathlib.Path(metadata["standard_bam"][ind]).is_file():
+                standard_bam = metadata["standard_bam"][ind]
             else:
                 standard_bam = None
         else:
             standard_bam = None
-            logger.info("genotype_variants::small_variants::multiple_patient:: Standard BAM file to genotype variants is not present.")
-        if pd.notnull(metadata['duplex_bam'][ind]):
-            if pathlib.Path(metadata['duplex_bam'][ind]).is_file():
-                duplex_bam = metadata['duplex_bam'][ind]
+            logger.info(
+                "genotype_variants::small_variants::multiple_samples:: Standard BAM file to genotype variants is not present."
+            )
+        if pd.notnull(metadata["duplex_bam"][ind]):
+            if pathlib.Path(metadata["duplex_bam"][ind]).is_file():
+                duplex_bam = metadata["duplex_bam"][ind]
             else:
                 duplex_bam = None
         else:
             duplex_bam = None
-        if pd.notnull(metadata['simplex_bam'][ind]):
-            if pathlib.Path(metadata['simplex_bam'][ind]).is_file():
-                simplex_bam = metadata['simplex_bam'][ind]
+        if pd.notnull(metadata["simplex_bam"][ind]):
+            if pathlib.Path(metadata["simplex_bam"][ind]).is_file():
+                simplex_bam = metadata["simplex_bam"][ind]
             else:
                 simplex_bam = None
         else:
             simplex_bam = None
-        if duplex_bam and simplex_bam:
-            logger.info("genotype_variants::small_variants::multiple_patient:: duplex_bam and simplex_bam are present for genotype variants.")
+
+        if standard_bam or duplex_bam or simplex_bam:
+            logger.info(
+                "genotype_variants::small_variants::multiple_samples:: standard_bam, duplex_bam and simplex_bam are present for genotype variants."
+            )
         else:
-            logger.error("genotype_variants::small_variants::multiple_patient:: duplex_bam and simplex_bam are not present for genotype variants! Please provide both of them to run genotype_variants.")
-            exit(1)
-        if pd.notnull(metadata['patient_id'][ind]):
-            patient_id = metadata['patient_id'][ind]
+            logger.warning(
+                "genotype_variants::small_variants::multiple_samples:: one of standard_bam, duplex_bam and simplex_bam is not present for genotype variants! Either the Standard BAM or the Duplex BAM and the Simplex BAM should be present for genotype variants."
+            )
+
+        if pd.notnull(metadata["sample_id"][ind]):
+            sample_id = metadata["sample_id"][ind]
         else:
-            logger.error("genotype_variants:small_variants:multiple_patient:: Patient Id is not a string, please check input metadata file and try again.")
+            logger.error(
+                "genotype_variants:small_variants:multiple_samples:: Sample id is not a string, please check input metadata file and try again."
+            )
             exit(1)
-        logger.info("genotype_variants:small_variants::multiple_patient:: %s is being processed", patient_id)
+        logger.info(
+            "genotype_variants:small_variants::multiple_samples:: %s is being processed",
+            sample_id,
+        )
         final_file = all.callback(
             input_maf,
             reference_fasta,
             gbcms_path,
-            patient_id,
+            sample_id,
             standard_bam,
             duplex_bam,
             simplex_bam,
